/* Server interface of parapov.
 * This modul contains all defines, typedefs and prototypes for 
 * parapov_mas.h. 
 *
 * These are structures to store the status 
 * inforamtion about each slave, frame and block:
 *   para_slave_stat
 *   para_frame_stat
 *   para_block_stat
 *
 * There are also some status indicators for each structure.
 * 
 * At least the prototypes of the funktions called by povray
 * are deklared here:
 *   para_master_init ()
 *   para_master_loop ()
 *   para_master_kill_slaves ()
 *
 * Datei: parapov_mas.h		Autor: Benjamin Goldbach
 * Datum: 14.11.2002		Version: 0.1
 *
 * Historie:
 *
 */
#ifndef __PARAPOV_MAS_H
#define __PARAPOV_MAS_H


/* status information for the para_slave_stat struct */
#define PARA_SLAVE_FREE		-1	/* if nothing is assigned	*/


/* Status indicators for the para_block_stat struct */
enum
{
  PARA_GRID_POINT_FREE = 0,		/* block is not assigned	*/
  PARA_GRID_POINT_ASSIGNED,		/* block assigned to a client	*/
  PARA_GRID_POINT_DONE,			/* block is rendered		*/
  PARA_GRID_POINT_WRITTEN		/* block is written to the file */
};


/*  Status indicators for the para_frame_stat struct */ 
enum
{
  PARA_FRAME_FREE = 0,			/* nothing is done for the frame*/
  PARA_FRAME_WORKING,			/* frame is currently rendered	*/
  PARA_FRAME_DONE,			/* all have been rednered	*/
  PARA_FRAME_DONE_SHELLOUT		/* frame is finished		*/
};


/* Structure to hold info about each slave process */
typedef struct
{
  struct info_s info;			/* information about the client	*/
  int	 pixels_done;			/* Number of pixels completet	*/
  int	 blocks_done;			/* Number of blocks completet	*/
  DBL	 trender;			/* Time needet for rendering	*/
  int	 frame_assigned;		/* frame currently working on	*/
} para_slave_stat;


/* Structure to hold info about each grid section */
typedef struct
{
  int status;				/* Current status of block	*/
  int wr_line;				/* Line to be written next	*/
  int assigned_tid;	/* Tid of the machinde assigned to this block	*/
} para_block_stat;


/* Structure to hold info about each frame */
typedef struct
{
  int status;				/* current status of the frame	*/
  COLOUR** row_ptr;			/* COLOUR data for one line	*/
  int row_to_write;			/* row to write next		*/
  int row_to_malloc;			/* next row to allocate mem for	*/
  int block_to_assign;			/* block to assign next		*/
  int block_to_write;			/* block to write next		*/
  COUNTER stats[MaxStat];		/* PovRay Status 		*/
  para_block_stat* block_stat;		/* info for the frames blocks	*/ 
  Image_File_Class* output_file;	/* Object for the render output	*/
} para_frame_stat;



/* Global variables */
extern int mytid;			/* my MPI task id		*/
extern int ntasks;			/* number of MPI tasks 		*/


void para_master_init ();
void para_master_kill_slaves ();
void para_master_loop ();

#endif // __PARAPOV_MAS_H
