/* The slave inteface
 *
 * File: parapov_slv.h		Author: Matthias Fueller
 * Date: 13.11.2002 		Version: 0.1
 *
 * Change log:
 */
#ifndef __PARAPOV_SLV_H__
#define __PARAPOV_SLV_H__


/* status returned by para_slave_get_work */
enum slave_status
{
  ST_NEW_BLOCK = 1,			/* new block received 		*/
  ST_NEW_FRAME,				/* new block, new frame		*/
  ST_EXIT				/* everything is over		*/
};


/* Class BufferImage
 *
 * This class is used to redirect the slave render output.
 * The data is stored in a buffer and sent to the master.
 */
class BufferImage : public Image_File_Class
{
  public:
    BufferImage ();
    ~BufferImage ();

    void init ();

    virtual void Write_Line (COLOUR *line_data);
    virtual int Read_Line (COLOUR *line_data);

    virtual int Line () { return line; }
    virtual int Width () { return width; }
    virtual int Height () { return height; }

  protected:
    COLOUR *buf;			/* buffer to store data		*/
    int width;				/* width of the block		*/
    int height;				/* height of the block		*/
    int line;				/* index of next line in buf	*/
    int last_line;			/* end of last sent block 	*/
    int lines_to_send;			/* number of lines to send	*/
};


/*
 * global variables
 */
extern int para_next_frame;		/* next frame to render 	*/
/* singleton object for the redirection buffer */
extern BufferImage *para_buffer_singleton;
extern DBL para_render_time;		/* render time of one block	*/


slave_status para_slave_get_work ();
void para_redirect_slave_output ();
void para_slave_init ();
void para_slave_exit (int i);
void para_slave_send_stats ();
void para_slave_send_error (const char *msg, int mpi_error);

#endif /* __PARAPOV_SLV_H__  */
